@@ -381,6 +381,39 @@ get.w.mnps <- function(mnps, stop.method = NULL, ...) {
     
     return(w)
 }
+get.w.iptw <- function(iptw, stop.method = NULL, ...) {
+    if (length(stop.method) > 0) {
+        if (any(is.character(stop.method))) {
+            rule1 <- names(iptw$psList[[1]]$ps)[sapply(names(iptw$psList[[1]]$ps), function(x) any(startsWith(tolower(x), tolower(stop.method))))]
+            if (length(rule1) == 0) {
+                message(paste0("Warning: stop.method should be ", word.list(names(iptw$psList[[1]]$ps), and.or = "or", quotes = TRUE), ".\nUsing all available stop methods instead."))
+                rule1 <- names(iptw$psList[[1]]$ps)
+            }
+        }
+        else if (is.numeric(stop.method) && any(stop.method %in% seq_along(names(iptw$psList[[1]]$ps)))) {
+            if (any(!stop.method %in% seq_along(names(iptw$psList[[1]]$ps)))) {
+                message(paste0("Warning: There are ", length(names(iptw$psList[[1]]$ps)), " stop methods available, but you requested ", 
+                               word.list(stop.method[!stop.method %in% seq_along(names(iptw$psList[[1]]$ps))], and.or = "and"),"."))
+            }
+            rule1 <- names(iptw$psList[[1]]$ps)[stop.method %in% seq_along(names(iptw$psList[[1]]$ps))]
+        }
+        else {
+            warning("stop.method should be ", word.list(names(iptw$psList[[1]]$ps), and.or = "or", quotes = TRUE), ".\nUsing all available stop methods instead.", call. = FALSE)
+            rule1 <- names(iptw$psList[[1]]$ps)
+        }
+    }
+    else {
+        rule1 <- names(iptw$psList[[1]]$ps)
+    }
+    
+    w <- setNames(as.data.frame(matrix(NA, nrow = nrow(iptw$psList[[1]]$ps),
+                                       ncol = length(rule1))),
+                  rule1)
+    for (i in rule1) {
+        w[i] <- Reduce("*", lapply(iptw$psList, function(x) get.w.ps(x, stop.method = i)))
+    }
+    return(w)
+}
 get.w.Match <- function(M,  ...) {
     nobs <- M$orig.nobs
     weights.list <- index.list <- setNames(vector("list", 4), c("control", "treated", "unmatched", "dropped"))
@@ -433,6 +466,9 @@ get.w.CBPS <- function(c, estimand = NULL, ...) {
 get.w.npCBPS <- function(c, estimand = NULL, ...) {
     return(c$weights)
 }
+get.w.CBMSM <- function(c, ...) {
+    return(c$weights)
+}
 get.w.ebalance <- function(e, treat, ...) {
     if (missing(treat)) stop("treat must be specified.", call. = FALSE)
     
@@ -445,7 +481,8 @@ get.w.ebalance <- function(e, treat, ...) {
     return(weights)
 }
 get.w.ebalance.trim <- get.w.ebalance
-get.w.optmatch <- function(o, treat, ...) {
+get.w.optmatch <- function(o, ...) {
+    treat <- as.numeric(attr(o, "contrast.group"))
     return(match.strata2weights(o, treat = treat, covs = NULL))
 }
 get.w.weightit <- function(W, ...) {
@@ -504,6 +541,14 @@ nunique.gt <- function(x, n) {
     if (length(x) < 2000) nunique(x) > n
     else tryCatch(nunique(x, nmax = n) > n, error = function(e) TRUE)
 }
+is.formula <- function(f, sides = NULL) {
+    res <- is.name(f[[1]])  && deparse(f[[1]]) %in% c( '~', '!') &&
+        length(f) >= 2
+    if (length(sides) > 0 && is.numeric(sides) && sides %in% c(1,2)) {
+        res <- res && length(f) == sides + 1
+    }
+    return(res)
+}
 
 #Under construction
 inxnoty <- function(x, y) {
